// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include "configuration.hpp"
#include "logger.hpp"

const char* cfg_proxyuser = "proxy.user";
const char* cfg_proxygroup = "proxy.group";
const char* cfg_listeners = "listeners";
const char* cfg_name = "name";
const char* cfg_host = "host";
const char* cfg_vendor = "vendor";
const char* cfg_ip = "ip";
const char* cfg_port = "port";
const char* cfg_target = "target";

Configuration::Configuration(char* a_Path)
{
    m_Cfg.loadFile(a_Path);
}

Configuration::~Configuration()
{
}

const char* Configuration::user()
{
    return (m_Cfg.lookup(cfg_proxyuser));
}

const char* Configuration::group()
{
    return (m_Cfg.lookup(cfg_proxygroup));
}

int Configuration::numListeners()
{
    Setting& listenergroup = m_Cfg.lookup(cfg_listeners);
    return listenergroup.getLength();
}

Listener* Configuration::newListener(int a_Index)
{
    Listener* listener = NULL;
    Setting& listenergroup = m_Cfg.lookup(cfg_listeners);
    int numListeners = listenergroup.getLength();
    const char* lName = NULL;	// Listener Name
    const char* lHost = NULL;	// Listener Host
    const char* lVendor = NULL;	// Listener Vendor
    const char* lIp = NULL;	// Listener IP
    int         lPort = 1723;	// Listener Port
    int         tPort = 1723;	// Target Port
    const char* tIp = NULL;	// Target IP

    if ((0 < a_Index) || (a_Index < numListeners))
    {
	int numSettings = listenergroup[a_Index].getLength();
	int setting;

	for (setting = 0; setting < numSettings; setting++)
	{
	    const char* settingname = listenergroup[a_Index][setting].getName();
	    if (!strncmp(cfg_name, settingname, strlen(settingname))) lName = listenergroup[a_Index][setting];
	    if (!strncmp(cfg_host, settingname, strlen(settingname))) lHost = listenergroup[a_Index][setting];
	    if (!strncmp(cfg_vendor, settingname, strlen(settingname))) lVendor = listenergroup[a_Index][setting];
	    if (!strncmp(cfg_ip, settingname, strlen(settingname))) lIp = listenergroup[a_Index][setting];
	    if (!strncmp(cfg_port, settingname, strlen(settingname))) lPort = listenergroup[a_Index][setting];
	    if (!strncmp(cfg_target, settingname, strlen(settingname)))
	    {
		Setting& listenerTarget = listenergroup[a_Index][setting];

		tIp = listenerTarget[cfg_ip];
		tPort = listenerTarget[cfg_port];
	    }
	}

	listener = new Listener(lIp, lPort);
	listener->setName(lName);
	listener->setHost(lHost);
	listener->setVendor(lVendor);
	listener->setTarget(tIp,tPort);
    }

    return listener;
}
