// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include <iostream>
#include <unistd.h>
#include <strings.h>
#include <arpa/inet.h>
#include <errno.h>

#include "entity.hpp"
#include "util.hpp"

list<Entity*> Entity::s_EntityList;

Entity::Entity(const int a_Socket)
    throw(MemoryException) :
    m_Socket(a_Socket),
    m_BufferSize(4096),
    m_BufferUse(0),
    m_Buffer(NULL),
    m_BytesRead(0),
    m_BytesWritten(0)
{
    m_Buffer = malloc(m_BufferSize);
    if (NULL == m_Buffer)
    {
	if (m_Socket) close(m_Socket);
	throw MemoryException(errno);
    }

    memset(&m_SockAddr4, 0, sizeof(m_SockAddr4));
    gettimeofday(&m_ConnectDate, NULL);
    s_EntityList.push_back(this);
    DEBUGF(("Entity::Entity = %p\n", this));
}

Entity::~Entity()
{
    if (m_Buffer) free(m_Buffer);
    if (m_Socket) close(m_Socket);
    s_EntityList.erase(find(s_EntityList.begin(), s_EntityList.end(), this));
    DEBUGF(("Entity::~Entity = %p\n", this));
}

int Entity::read(void* a_Buffer, const int a_Length)
{
    m_BufferUse = recvfrom(m_Socket, a_Buffer, a_Length, 0, NULL, NULL);
    if (m_BufferUse > 0)
    {
	gettimeofday(&m_UpdateDate, NULL);
	m_BytesRead += m_BufferUse;
    }

    return m_BufferUse;
}

int Entity::write(void* a_Buffer, const int a_Length)
{
    int bytesWritten;

    bytesWritten = send(m_Socket, a_Buffer, a_Length, 0);
    if (bytesWritten > 0) m_BytesWritten += bytesWritten;

    return bytesWritten;
}
