// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#ifndef __ENTITY_HPP
#define __ENTITY_HPP
#include <list>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>

#include "exception.hpp"

using std::list;

class Entity
{
    friend class Session;
    friend class Call;
    public:
	Entity(const int a_Socket)
	    throw(MemoryException);
	virtual ~Entity();

	static list<Entity*> entityList() { return s_EntityList; };

	int read() { return read(m_Buffer, m_BufferSize); };
	int read(const int a_Length) { return read(m_Buffer, a_Length); };
	int read(void* a_Buffer, const int a_Length);
	int write() { return write(m_Buffer, m_BufferUse); };
	int write(const int a_Length) { return write(m_Buffer, a_Length); };
	int write(void* a_Buffer, const int a_Length);

	virtual int process() = 0;
	virtual Session* detatch() = 0;

	const int m_Socket;
	Session* m_Session;

    protected:
	const int m_BufferSize;
	int m_BufferUse;
	void* m_Buffer;

	struct sockaddr_in m_ServerAddr4;
	struct sockaddr_in6 m_ServerAddr6;
	struct sockaddr_in m_SockAddr4;
	struct sockaddr_in6 m_SockAddr6;
	struct sockaddr_in m_TargetAddr4;
	struct sockaddr_in6 m_TargetAddr6;

	struct timeval m_ConnectDate;
	struct timeval m_UpdateDate;
	unsigned long m_BytesRead;
	unsigned long m_BytesWritten;

    private:
	static list<Entity*> s_EntityList;
};

#endif
