// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#ifndef __EXCEPTION_HPP
#define __EXCEPTION_HPP
#include <stdio.h>
#include <string.h>

class GeneralException
{
    protected:
	const char* m_Message;

    public:
	GeneralException() :
	    m_Message(NULL) {};
	GeneralException(int a_Errno) { m_Message = strerror(a_Errno); };
	GeneralException(const char* a_Message) :
	    m_Message(a_Message) {};
	virtual ~GeneralException() {};

	virtual const char* errorMessage() { return m_Message; };
};

class MemoryException : public GeneralException
{
    public:
	MemoryException() {};
	MemoryException(int a_Errno) : GeneralException(a_Errno) {};
	MemoryException(const char* a_Message) : GeneralException(a_Message) {};
	~MemoryException() {};
};

class NetworkException : public GeneralException
{
    public:
        NetworkException() {};
	NetworkException(int a_Errno) : GeneralException(a_Errno) {};
	NetworkException(const char* a_Message) : GeneralException(a_Message) {};
	~NetworkException() {};
};

class PrivdropException
{
    private:
	const char* m_User;
	const char* m_Group;

    public:
	PrivdropException() :
	    m_User(NULL), m_Group(NULL) {};
	PrivdropException(const char* a_User, const char* a_Group) :
	    m_User(a_User), m_Group(a_Group) {};
	~PrivdropException() {};

	const char* user() { return m_User; };
	const char* group() { return m_Group; };
};

#endif
