// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include <stdlib.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <errno.h>

#include "grehandler.hpp"
#include "util.hpp"

GreHandler::GreHandler()
throw(NetworkException, MemoryException)
    : m_BufferSz(4096)
{
    if (0 < (m_GreSocket = getSocket(SOCK_RAW, IPPROTO_GRE)))
    {
	int options = 0;
	if (setsockopt(m_GreSocket, 0, IP_HDRINCL, &options, sizeof(options))) throw NetworkException(errno);
    }
    else
    {
	throw NetworkException(errno);
    }

    if (0 < (m_RawSocket = getSocket(SOCK_RAW, IPPROTO_RAW)))
    {
	int options = 1;
	if (setsockopt(m_RawSocket, 0, IP_HDRINCL, &options, sizeof(options))) throw NetworkException(errno);
    }
    else
    {
	throw NetworkException(errno);
    }

    if (NULL == (m_Buffer = malloc(m_BufferSz))) throw MemoryException(errno);
    m_IpHeader = reinterpret_cast<struct IpHeader*>(m_Buffer);
    m_GreHeader = reinterpret_cast<struct GreHeader*>(static_cast<char*>(m_Buffer) + sizeof(struct IpHeader));
}

GreHandler::~GreHandler()
{
    if (m_RawSocket) close(m_RawSocket);
    if (m_GreSocket) close(m_GreSocket);
    if (m_Buffer) free(m_Buffer);
}

int GreHandler::recvFrom(struct sockaddr_in* a_SourceAddr, u_int16_t& a_CallID, int& a_Length)
{
    socklen_t sockLen = sizeof(struct sockaddr_in);
    a_Length = recvfrom(m_GreSocket, m_Buffer, m_BufferSz, 0, reinterpret_cast<struct sockaddr*>(a_SourceAddr), &sockLen);
    a_CallID = m_GreHeader->gh_CallID;
    return (-1 == a_Length);
}

int GreHandler::sendTo(struct sockaddr_in* a_DestAddr, u_int16_t a_CallID, int& a_Length)
{
    m_GreHeader->gh_CallID = a_CallID;
    a_Length = sendto(m_GreSocket, static_cast<char*>(m_Buffer) + sizeof(struct IpHeader), a_Length - sizeof(struct IpHeader), 0, reinterpret_cast<struct sockaddr*>(a_DestAddr), sizeof(struct sockaddr_in));
    return (-1 == a_Length);
}

int GreHandler::sendTo(struct sockaddr_in* a_SourceAddr, struct sockaddr_in* a_DestAddr, u_int16_t a_CallID, int& a_Length)
{
#if defined(__OpenBSD__)
    m_IpHeader->ih_PacketLen = htons(a_Length);
#elif defined(__FreeBSD__)
    m_IpHeader->ih_PacketLen = a_Length;
#else
    m_IpHeader->ih_PacketLen = a_Length;
#endif
    m_IpHeader->ih_SourceAddr = a_SourceAddr->sin_addr.s_addr;
    m_IpHeader->ih_DestAddr = a_DestAddr->sin_addr.s_addr;
    m_GreHeader->gh_CallID = a_CallID;
    a_Length = sendto(m_RawSocket, m_Buffer, a_Length, 0, reinterpret_cast<struct sockaddr*>(a_DestAddr), sizeof(struct sockaddr_in));
    return (-1 == a_Length);
}
