#ifndef _GREHANDLER_HPP
#define _GREHANDLER_HPP
#include <unistd.h>

#include "exception.hpp"

struct IpHeader
{
    u_int8_t ih_HeaderLen; // Lower 4 bits
    u_int8_t ih_Tos;
    u_int16_t ih_PacketLen;
    u_int16_t ih_Id;
    u_int16_t ih_FragInfo;
    u_int8_t ih_Ttl;
    u_int8_t ih_Protocol;
    u_int16_t ih_Checksum;
    u_int32_t ih_SourceAddr;
    u_int32_t ih_DestAddr;
} __attribute__((__packed__));

struct GreHeader
{
    u_int16_t gh_Flags;
    u_int16_t gh_Type;
    u_int16_t gh_Length;
    u_int16_t gh_CallID;
    u_int32_t gh_SequenceNr;
    u_int32_t gh_AckNr;
} __attribute__((__packed__));

class GreHandler
{
    public:
	GreHandler() throw(NetworkException, MemoryException);
	~GreHandler();

	int recvFrom(struct sockaddr_in* a_SourceAddr, u_int16_t& a_CallID, int& a_Length);
	int sendTo(struct sockaddr_in* a_DestAddr, u_int16_t a_CallID, int& a_Length);
	int sendTo(struct sockaddr_in* a_SourceAddr, struct sockaddr_in* a_DestAddr, u_int16_t a_CallID, int& a_Length);

	int greSocket() { return m_GreSocket; };
	int rawSocket() { return m_RawSocket; };

    private:
	void* m_Buffer;
	int m_BufferSz;
	int m_GreSocket;
	int m_RawSocket;
	struct IpHeader* m_IpHeader;
	struct GreHeader* m_GreHeader;
};

#endif
