// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>

#include "listener.hpp"
#include "logger.hpp"
#include "util.hpp"

Listener::Listener(const char* a_Address, const int a_Port)
    throw(NetworkException)
    : m_Socket(0), m_Address(a_Address), m_Port(a_Port),
    m_Name(NULL), m_Host(NULL), m_Vendor(NULL)
{
    int* socket = const_cast<int*>(&m_Socket);

    memset(&m_SockAddr4, 0, sizeof(m_SockAddr4));
    m_SockAddr4.sin_family = AF_INET;
    m_SockAddr4.sin_port = htons(a_Port);
    if (a_Address && 0 == inet_aton(a_Address, &m_SockAddr4.sin_addr)) throw NetworkException(errno);
    if (0 == (*socket = getSocket(SOCK_STREAM, IPPROTO_IP))) throw NetworkException(errno);
    if (-1 == bind(m_Socket, reinterpret_cast<struct sockaddr*>(&m_SockAddr4), sizeof(m_SockAddr4)))
    {
	close(m_Socket);
	throw NetworkException(errno);
    }
    if (-1 == listen(m_Socket, 0))
    {
	close(m_Socket);
	throw NetworkException(errno);
    }
}

Listener::~Listener()
{
    close(m_Socket);
}

void Listener::setTarget(const char* a_Address, int a_Port)
{
    m_Target.address = a_Address;
    m_Target.port = a_Port;

    memset(&m_Target.sockAddr4, 0, sizeof(struct sockaddr_in));
    m_Target.sockAddr4.sin_family = AF_INET;
    m_Target.sockAddr4.sin_port = htons(a_Port);
    if (a_Address && 0 == inet_aton(a_Address, &m_Target.sockAddr4.sin_addr));
}

int Listener::accept()
{
    int socket;
    struct sockaddr_in sockAddr4;
    socklen_t sockLen = sizeof(sockAddr4);

    socket = ::accept(m_Socket, reinterpret_cast<struct sockaddr*>(&sockAddr4), &sockLen);
    Logger::getLogger()->log(LOG_INFO, "New connection from %s:%u",
	    inet_ntoa(sockAddr4.sin_addr), sockAddr4.sin_port);
    return socket;
}
