// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include <stdarg.h>
#include <syslog.h>

#include "logger.hpp"
#include "exception.hpp"

Logger* Logger::s_Logger = NULL;

Logger::Logger(const char* a_Identity, unsigned int a_Facility, unsigned int a_Flags)
   throw(GeneralException) :
	m_Flags(a_Flags)
{
   // Verify that only one instance of the logger is running.
   if (s_Logger) throw GeneralException();

   s_Logger = this;
   openlog(a_Identity, 0, a_Facility);
}

Logger::~Logger()
{
   closelog();
   s_Logger = NULL;
}

// Output a message to various log entities.  The message should
// not contain ending newline character
void Logger::log(unsigned int a_Level, char* a_Format, ...)
{
   va_list ap;

   if ((a_Level == LOG_DEBUG) && !(m_Flags & LOGGER_DEBUG)) return;

   if (m_Flags & LOGGER_SYSLOG)
   {
       va_start(ap, a_Format);
       vsyslog(a_Level, a_Format, ap);
       va_end(ap);
   }

   if (m_Flags & LOGGER_STDOUT)
   {
       va_start(ap, a_Format);
       vprintf(a_Format, ap);
       va_end(ap);
       printf("\n");
   }
}
