// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/pfvar.h>
#include <errno.h>

#include <arpa/inet.h>

#include "nat.hpp"
#include "logger.hpp"

const char* pfDevice = "/dev/pf";
int nat_PfDev = 0;

void natInit()
    throw(GeneralException)
{
   // Open /dev/pf for future nat lookups for all dynamic sessions.
   // The device must be opened for both reading and writing as root
   nat_PfDev = open(pfDevice, O_RDWR);
   if (nat_PfDev == -1)
   {
       nat_PfDev = 0;
       Logger::getLogger()->log(LOG_ERR, "Unable to open '%s' device, terminating!", pfDevice);
       throw GeneralException(errno);
   }
}

void natClose()
{
    if (nat_PfDev)
    {
	close(nat_PfDev);
	nat_PfDev = 0;
    }
}

int natLook4(const int a_Socket, const struct sockaddr_in& a_LookupAddr4, struct sockaddr_in& a_ResultAddr4)
    throw(NetworkException)
{
    struct sockaddr_in proxyAddr4;
    struct pfioc_natlook natlook;
    socklen_t addrlen;
    int ret = 0;

    addrlen = sizeof(proxyAddr4);
    if (getsockname(a_Socket, reinterpret_cast<struct sockaddr*>(&proxyAddr4), &addrlen) != 0)
    {
	throw NetworkException(errno);
    }

    // Set up the natlook struct with what we know of
    // the client, source ip:port and target ip:host.
    memset(static_cast<void*>(&natlook), 0, sizeof(natlook));
    natlook.af = AF_INET;
    natlook.saddr.addr32[0] = a_LookupAddr4.sin_addr.s_addr;
    natlook.daddr.addr32[0] = proxyAddr4.sin_addr.s_addr;
    natlook.proto = IPPROTO_TCP;
    natlook.sport = a_LookupAddr4.sin_port;
    natlook.dport = proxyAddr4.sin_port;
    natlook.direction = PF_OUT;

    if (ioctl(nat_PfDev, DIOCNATLOOK, &natlook) == -1)
    {
	ret = errno;
	if (ENOENT == errno)
	{
	    // Getting here means that we were unable to look up the original
	    // target.  If the PF rules are setup as specified in the documents
	    // this should mean that the client has connected to one of the
	    // local interfaces, eg the proxy itself.
	    natlook.rdport = proxyAddr4.sin_port;
	    natlook.rdaddr.addr32[0] = proxyAddr4.sin_addr.s_addr;
	}
	else
	{
	    throw NetworkException(errno);
	}
    }

    // After a successful nat lookup we now know the original
    // intended target (real destination) ip and port.
    memset(static_cast<void*>(&a_ResultAddr4), 0, sizeof(struct sockaddr_in));
    a_ResultAddr4.sin_port = natlook.rdport;
    a_ResultAddr4.sin_addr.s_addr = natlook.rdaddr.addr32[0];
    a_ResultAddr4.sin_len = sizeof(struct sockaddr_in);
    a_ResultAddr4.sin_family = AF_INET;

    return ret;
}
