// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#ifndef __RFC2637_HPP
#define __RFC2637_HPP

/* RFC2637 - Point-to-Point Tunneling Protocol (PPTP)*/

#define MAGIC_COOKIE 0x1A2B3C4D

/* PPTP control message types [RFC2637 1.4] */
enum ctrl_types {PPTP_CTRL = 1, PPTP_MGMT};

/* PPTP control messages [RFC2637 1.4] */
enum ctrl_messages {SCCRQ = 1, SCCRP, TCCRQ, TCCRP, ECORQ, ECORP, OCRQ, OCRP, ICRQ, ICRP, ICCN, CCRQ, CDN, WEN, SLI};

/* General error codes */
enum general_errors {NONE = 0, NOT_CONNECTED, BAD_FORMAT, BAD_VALUE, NO_RSRC, BAD_CALL_ID, PAC_ERROR};

/* PPTP Message "Header" */
struct pptp_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
} __attribute__((__packed__));

/* Start-Control-Connection-Request [RFC2637 2.1] */
struct sccrq_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int8_t protocolHi;
    u_int8_t protocolLo;
    u_int16_t reserved1;
    u_int32_t framingCap;
    u_int32_t bearerCap;
    u_int16_t maxChannels;
    u_int16_t firmwareRev;
    char hostname[64];
    char vendor[64];
} __attribute__((__packed__));

/* Start-Control-Connection result codes [RFC2637 2.1] */
enum sccrp_results {SCC_OK = 1, SCC_ERROR, SCC_EXIST, SCC_AUTH, SCC_VER};

/* Start-Control-Connection-Reply */
struct sccrp_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int8_t protocolHi;
    u_int8_t protocolLo;
    u_int8_t result;
    u_int8_t error;
    u_int32_t framingCap;
    u_int32_t bearerCap;
    u_int16_t maxChannels;
    u_int16_t firmwareRev;
    char hostname[64];
    char vendor[64];
} __attribute__((__packed__));

/* Stop-Control-Connection-Request */
struct tccrq_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int8_t reason;
    u_int8_t reserved1;
    u_int16_t reserved2;
} __attribute__((__packed__));

/* Start-Control-Connection result codes [RFC2637 2.4] */
enum tccrp_results {TCC_OK = 1, TCC_ERROR};

/* Stop-Control-Connection-Reply */
struct tccrp_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int8_t result;
    u_int8_t error;
    u_int16_t reserved1;
} __attribute__((__packed__));

/* Echo-Request */
struct ecorq_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int32_t identifier;
} __attribute__((__packed__));

/* Echo-Reply result codes */
enum eco_results {ECO_OK = 1, ECO_ERROR};

/* Echo-Reply */
struct ecorp_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int32_t identifier;
    u_int8_t result;
    u_int8_t error;
    u_int16_t reserved1;
} __attribute__((__packed__));

/* Outgoing-Call-Request */
struct ocrq_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int16_t callId;
    u_int16_t callSerial;
    u_int32_t minBps;
    u_int32_t maxBps;
    u_int32_t bearerType;
    u_int32_t framingType;
    u_int16_t recvWinSz;
    u_int16_t procDelay;
    u_int16_t numberLength;
    u_int16_t reserved1;
    char phoneNumber[64];
    char subaddress[64];
} __attribute__((__packed__));

/* Outgoing-Call result codes */
enum occ_results {OC_OK = 1, OC_ERROR, OC_CARRIER, OC_BUSY, OC_TONE, OC_TIMEOUT, OC_DENIED};

/* Outgoing-Call-Reply */
struct ocrp_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int16_t callId;
    u_int16_t peerCallId;
    u_int8_t result;
    u_int8_t error;
    u_int16_t cause;
    u_int32_t speed;
    u_int16_t recvWinSz;
    u_int16_t procDelay;
    u_int32_t channelId;
} __attribute__((__packed__));

/* Incoming-Call-Request */
struct iccrq_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
} __attribute__((__packed__));

/* Incoming-Call-Reply */
struct iccrp_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
} __attribute__((__packed__));

/* Incoming-Call-Connected */
struct icc_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
} __attribute__((__packed__));

/* Call-Clear-Request */
struct ccrq_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int16_t callId;
    u_int16_t reserved1;
} __attribute__((__packed__));

enum cdn_results {CDN_CARRIER = 1, CDN_ERROR, CDN_SHUTDOWN, CDN_REQUEST};

/* Call-Disconnect-Notify */
struct cdn_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int16_t callId;
    u_int8_t result;
    u_int8_t error;
    u_int16_t cause;
    u_int16_t reserved1;
    char stats[128];
} __attribute__((__packed__));

/* WAN-Error-Notify */
struct wen_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magic_cookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int16_t peerCallId;
    u_int16_t reserved1;
    u_int32_t crcErrorrs;
    u_int32_t framingErrors;
    u_int32_t hardwareOverruns;
    u_int32_t bufferOverruns;
    u_int32_t timeoutErrors;
    u_int32_t alignmentErrors;
} __attribute__((__packed__));

/* Set-Link-Info */
struct sli_msg
{
    u_int16_t length;
    u_int16_t type;
    u_int32_t magicCookie;
    u_int16_t ctrlType;
    u_int16_t reserved0;
    u_int16_t peerCallId;
    u_int16_t reserved1;
    u_int32_t sendAccm;
    u_int32_t receiveAccm;
} __attribute__((__packed__));


/* Prototypes */
int build_SCCRQ(void* a_Buffer, int a_ProtocolHi, int a_ProtocolLo, int a_FramingCap,
	int a_BearerCap, int a_MaxChannels, int a_FirmwareRev,
	char* a_Hostname, char* a_Vendor);
int build_SCCRP(void* a_Buffer, int a_ProtocolHi, int a_ProtocolLo, int a_Result, int a_Error,
	unsigned int a_FramingCap, unsigned int a_BearerCap, int a_MaxChannels,
	int a_FirmwareRev, char* a_Hostname, char* a_Vendor);

int build_ECORQ(void* a_Buffer, int a_Identifier);
int build_ECORP(void* a_Buffer, int a_Identifier, int a_Result, int a_Error);

int build_OCRQ(void* a_Buffer, int a_CallId, unsigned int a_MinBps, unsigned int a_MaxBps,
	unsigned int a_BearerType, unsigned int a_FramingType, int a_RecvWinSz,
	int a_ProcDelay, int *a_NumberLength, char *a_PhoneNumber, char *a_Subaddress);
int build_OCRP(void* a_Buffer, int a_CallId, int a_PeerCallId, int a_Result, int a_Error,
	int a_Cause, unsigned int a_Speed, int a_RecvWinSz, int a_ProcDelay, unsigned int a_ChannelId);

int build_TCCRP(void* a_Buffer, int a_Result, int a_Error);

int build_CCRQ(void* a_Buffer, int a_CallId);

int build_CDN(void* a_Buffer, int a_CallId, int a_Result, int a_Error, int a_Cause, char* a_Stats);
#endif
