// Frickin PPTP Proxy, Copyright (c) 2004-06 Martin Akesson <@placid.tv>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#ifndef __SESSION_HPP
#define __SESSION_HPP

#include <sys/poll.h>

#include "call.hpp"
#include "exception.hpp"

using std::list;

class Client;
class Server;

class Session
{
    public:
	Session(Server* a_Server, Client* a_Client);
	~Session();

	static int getSession(const int a_Socket, Session*& a_Session);

	void attatch(Client* a_Client);
	Session* detatch(Client* a_Client);
	void connectServer() throw(NetworkException);
	int getEntityList(Entity**& a_EntityList, int& a_ListSize);

	int process(Server* a_Server);
	int process(Client* a_Client);

	void setHostname(const char* a_Name) { memcpy(&m_Hostname, a_Name, 64); };
	void setVendor(const char* a_Vendor) { memcpy(&m_Vendor, a_Vendor, 64); };

	bool operator==(const Server* a_Server);

    private:
	Server* m_Server;
	list<Client*> m_ClientList;
	list<Call*> m_CallList;

	char m_Hostname[64];
	char m_Vendor[64];
	int m_FramingCap;
	int m_BearerCap;
	int m_MaxChannels;
	int m_FirmwareRev;

	static list<Session*> s_SessionList;
};

#endif
